package eu.chorevolution.vsb.gm.protocols.rest;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.restlet.Client;
import org.restlet.Component;
import org.restlet.Request;
import org.restlet.Response;
import org.restlet.Restlet;
import org.restlet.Server;
import org.restlet.data.Header;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Protocol;
import org.restlet.data.Status;
import org.restlet.engine.header.HeaderConstants;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.Get;
import org.restlet.resource.ResourceException;
import org.restlet.resource.ServerResource;
import org.restlet.util.Series;

import eu.chorevolution.vsb.gm.protocols.primitives.BcGmSubcomponent;
import eu.chorevolution.vsb.gmdl.utils.BcConfiguration;
import eu.chorevolution.vsb.gmdl.utils.Data;
import eu.chorevolution.vsb.gmdl.utils.GmServiceRepresentation;
import eu.chorevolution.vsb.gmdl.utils.Operation;
import eu.chorevolution.vsb.gmdl.utils.Scope;
import eu.chorevolution.vsb.gmdl.utils.enums.OperationType;


public class BcRestSubcomponent extends BcGmSubcomponent {

	private Client client = null;
	private Server server = null;
	private Component component;

	private Component printerComponent;
	private Server printerServer;
	private static String printermsg = "empty";
	private static GmServiceRepresentation serviceRepresentation;
    private static BcRestSubcomponent bcRestSubcomponent;
//    private MeasureAgent agentPost = null;
//    private static MeasureAgent agentMget = null;
    private ExecutorService executor = null;

	public BcRestSubcomponent(

			BcConfiguration bcConfiguration, GmServiceRepresentation serviceRepresentation

	){

		super(bcConfiguration);
		
		this.bcRestSubcomponent = this; 		
		this.serviceRepresentation = serviceRepresentation;
		int restservicePort = Integer.valueOf(bcConfiguration.getServicePort());
		
		switch (this.bcConfiguration.getSubcomponentRole()){
		
		case SERVER:
			
//			agentMget = new MeasureAgent("timestamp_4",System.currentTimeMillis(),MonitorConstant.M4,MonitorConstant.timestamp_4_port_listener);
			this.server = new Server(Protocol.HTTP, this.bcConfiguration.getSubcomponentAddress(),this.bcConfiguration.getSubcomponentPort());
			this.component = new Component();
			this.component.getServers().add(server);
			this.component.getContext().getParameters().add("maxThreads", "100");
			this.component.getContext().getParameters().add("maxTotalConnections", "-1");
			break;
			
		case CLIENT:
			
			executor = Executors.newFixedThreadPool(10);
//			agentPost = new MeasureAgent("timestamp_3",System.currentTimeMillis(),MonitorConstant.M2,MonitorConstant.timestamp_3_port_listener);
			client = new Client(Protocol.HTTP);
			this.printerServer = new Server(Protocol.HTTP, restservicePort);
			this.printerComponent = new Component();
			this.printerComponent.getServers().add(printerServer);
			break;
			
		default:
			break;
		}
	}

	@Override
	public void start() {
		switch (this.bcConfiguration.getSubcomponentRole()){
		case SERVER:
			
			try {
				this.component.start();
			} catch (Exception e) {
				e.printStackTrace();
			}
			// this.component.setDefaultHost(this.bcConfiguration.getSubcomponentAddress());
			//this.component.getDefaultHost().attach("/", RestServerResource.class);
			this.component.getDefaultHost().attach("/", new RestletRestService());
			
			break;
		case CLIENT:
			try {

				this.client.start();
				this.printerComponent.start();

			} catch (Exception e1) {
				e1.printStackTrace();
			}
			this.printerComponent.getDefaultHost().attach("/getmessage", PrinterRestServerResource.class);
			break;
		default:
			break;
		}
	}

	@Override
	public void stop() {
		switch (this.bcConfiguration.getSubcomponentRole()) {
		case SERVER:
			try {

				this.component.stop();

			} catch (Exception e1) {

				e1.printStackTrace();
			}
			break;
		case CLIENT:
			try {
				this.client.stop();
				this.printerComponent.stop();
			} catch (Exception e) {
				e.printStackTrace();
			}
			break;
		default:
			break;
		}
	}

	@Override
	public void postOneway(final String destination, final Scope scope, final List<Data<?>> datas, final long lease) {
		
		
		JSONObject msgObj = new JSONObject();
		String message_id = "";
		for(Data d : datas){
			
			if(!d.getName().equals("message_id")){
				
				msgObj.put(d.getName(), d.getObject().toString());
				
			}else{
				
				message_id = d.getObject().toString();
			}
			
		}
		String message = msgObj.toJSONString()+"-"+message_id;
//		SendMessage sendMessage = new SendMessage(message);
//		agentPost.fire(""+System.currentTimeMillis()+"-"+message_id);
//		sendMessage.start();
//		agentPost.fire(""+System.currentTimeMillis()+"-"+message_id);
		executor.execute(new Runnable(){
			
			@Override
			public void run() {
				// TODO Auto-generated method stub
				
				Request request = new Request();
				request.setResourceRef(bcConfiguration.getServiceAddress());
				request.setMethod(Method.POST);
				request.setEntity(message, MediaType.APPLICATION_JSON);
				client.handle(request);
			}
		});
		
		
		
	}

	@Override
	public void mgetOneway(final Scope scope, final Object exchange){
		
		//agentMget.fire();
		this.nextComponent.postOneway(this.bcConfiguration.getServiceAddress(), scope, (List<Data<?>>) exchange, 0);
		
	}

	@Override
	public void xmgetOneway(final String source, final Scope scope, final Object exchange){

		this.nextComponent.postOneway(this.bcConfiguration.getServiceAddress(), scope, (List<Data<?>>) exchange, 0);
	}

	@Override
	public <T> T postTwowaySync(final String destination, final Scope scope, final List<Data<?>> datas,
			final long lease) {

		Request request = null;
		switch (scope.getVerb()) {
		case GET:
			
			request = RestRequestBuilder.buildRestGetRequest(destination,scope, datas);
			
			break;
		
		case POST:
			
			request = RestRequestBuilder.buildRestPostRequest(destination,scope, datas);
			
			break;

		default:
			
			printermsg = "Undefined verb";
			
			break;
		}
		Response response = this.client.handle(request);

		try {

				Thread.sleep(1000);

		} catch (InterruptedException e){
			
			e.printStackTrace();
		}
		printermsg = (response.getEntityAsText());

		return (T) response.getEntityAsText();
	}

	@Override
	public void xtgetTwowaySync(final String destination, final Scope scope, final long timeout,
			final Object response) {
		// TODO Auto-generated method stub
	}

	@Override
	public <T> T mgetTwowaySync(final Scope scope, final Object exchange) {

		return this.nextComponent.postTwowaySync(this.bcConfiguration.getServiceAddress(), scope,
				(List<Data<?>>) exchange, 0);
	}

	@Override
	public void postTwowayAsync(final String destination, final Scope scope, final List<Data<?>> data,
			final long lease) {
		// TODO Auto-generated method stub
	}

	@Override
	public void xgetTwowayAsync(final String destination, final Scope scope, final Object response) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mgetTwowayAsync(final Scope scope, final Object exchange) {
		this.nextComponent.postTwowayAsync(this.bcConfiguration.getServiceAddress(), scope, (List<Data<?>>) exchange,
				0);
	}

	@Override
	public void postBackTwowayAsync(final String source, final Scope scope, final Data<?> data, final long lease,
			final Object exchange) {
		// TODO Auto-generated method stub
	}

	public static class PrinterRestServerResource extends ServerResource {
		@Override
		protected Representation post(Representation entity) throws ResourceException {
			return new StringRepresentation("empty");
		}

		@Override
		protected Representation get() throws ResourceException {
			return new StringRepresentation(BcRestSubcomponent.printermsg);
		}

	}
	
	
	public class RestletRestService extends Restlet {

		private ExecutorService executor = null;

		public RestletRestService() {

			executor = Executors.newFixedThreadPool(10);
		}

		@Override
		public void handle(Request request, Response response) {

			if (request.getMethod().equals(Method.POST)) {

				try {
					String receivedText = request.getEntity().getText().toString();
					
					processing(receivedText);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				response.setAutoCommitting(false);
				final Response resp = response;
				response.setAutoCommitting(false);
				
				executor.execute(new Runnable() {

					@Override
					public void run() {

						resp.setStatus(Status.SUCCESS_OK);
						// resp.setEntity("OK Receive:)",
						// MediaType.TEXT_PLAIN);
						// resp.commit();
						return;
					}
				});
			}

		}

	}
	
	
	
	

	public static class RestServerResource extends ServerResource {

		
		@Override
		protected Representation post(Representation entity) throws ResourceException {
			String receivedText = null;
			try {
				receivedText = entity.getText();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			
//			System.err.println(receivedText);
			bcRestSubcomponent.processing(receivedText);
			return new StringRepresentation("");
		
		}

		@Get
		public org.restlet.Response receiveGet() {
			String remainingPart = getReference().getRemainingPart();
			Request req = getRequest();

			// TODO: Remove source. Not needed
			String source = req.getHostRef().getHostDomain();
			int port = req.getHostRef().getHostPort();

			Request request = getRequest();
			Map<String, Object> map = request.getAttributes();
			Object headers = map.get(HeaderConstants.ATTRIBUTE_HEADERS);
			int connectionTimeout = 60000;// Default value
			String callback = null;
			try {
				@SuppressWarnings("unchecked")
				Series<Header> headerSeries = (Series<Header>) headers;
				Header timeoutHeader = headerSeries.getFirst("connectionTimeout");
				if (timeoutHeader != null) {
					connectionTimeout = Integer.valueOf(timeoutHeader.getValue());
				}
				Header callbackHeader = headerSeries.getFirst("callbackURL");
				if (callbackHeader != null) {
					callback = callbackHeader.getValue();
				}
			} catch (ClassCastException e) {
				// GLog.log.e(TAG, "Class cast exeption: " + e);
			} catch (NumberFormatException nfe) {
				// GLog.log.e(TAG, "Class cast exeption: " + nfe);
			}

			// Message_CS msg = new Message_CS("", 0, source, port, "GET",
			// remainingPart, "", connectionTimeout, "", 0);
			//
			// fr.inria.arles.lsb.commons.Response lsbResponse = null;
			//
			// if (callback == null) {
			// lsbResponse = connectorRef.invokeSync(msg);
			// } else {
			// lsbResponse = connectorRef.invokeAsync(msg);
			// }

			org.restlet.Response restletResponse = null;
			// restletResponse = getResponse();getResponse();
			// restletResponse.setStatus(lsbResponse.get_respStatus());
			// restletResponse.setEntity(new
			// StringRepresentation(lsbResponse.getDataString()));
			return restletResponse;
		}


	}
	
	
	public void processing(String receivedText){

		JSONParser parser = new JSONParser();
		JSONObject jsonObject = null;
		String message_id = receivedText.split("-")[1];
		String message = receivedText.split("-")[0];
		try {
			jsonObject = (JSONObject) parser.parse(message);
//			agentMget.fire(""+System.currentTimeMillis()+"-"+message_id);
		} catch (ParseException e) {
			e.printStackTrace();
		}

		String op_name = (String) jsonObject.get("op_name");
		for (Entry<String, Operation> en : serviceRepresentation.getInterfaces().get(0).getOperations()
				.entrySet()) {
			if (en.getKey().equals(op_name)) {
				Operation op = en.getValue();
				List<Data<?>> datas = new ArrayList<>();

				for (Data<?> data : op.getGetDatas()) {
					Data d = new Data<String>(data.getName(), "String", true,
							(String) jsonObject.get(data.getName()), data.getContext());
					datas.add(d);
//					System.err.println("Added " + d);
				}
				Data d = new Data<String>("op_name", "String", true, op_name, "BODY");
				datas.add(d);
				d = new Data<String>("message_id", "String", true, message_id, "BODY");
				datas.add(d);
				if (op.getOperationType() == OperationType.TWO_WAY_SYNC) {

					String response = bcRestSubcomponent.mgetTwowaySync(op.getScope(), datas);

				} else if (op.getOperationType() == OperationType.ONE_WAY) {

					bcRestSubcomponent.mgetOneway(op.getScope(), datas);
				}
			}
		}

	}
	
	
	public class SendMessage extends Thread{
		
		String message = null;
		String scope = null;
		public SendMessage(String message){
			
			this.message = message;
		}
		
		public void run(){
			
			Request request = new Request();
			request.setResourceRef(bcConfiguration.getServiceAddress());
			request.setMethod(Method.POST);
			request.setEntity(message, MediaType.APPLICATION_JSON);
			client.handle(request);
		}
		
	}
}
